/*
	Copyright (C) 2008 Timo Bingmann, tb at panthema dot net
	Copyright (C) 2014 RoLex, webmaster at feardc dot net

	Verlihub is free software; You can redistribute it
	and modify it under the terms of the GNU General
	Public License as published by the Free Software
	Foundation, either version 3 of the license, or at
	your option any later version.

	Verlihub is distributed in the hope that it will be
	useful, but without any warranty, without even the
	implied warranty of merchantability or fitness for
	a particular purpose. See the GNU General Public
	License for more details.

	Please see http://www.gnu.org/licenses/ for a copy
	of the GNU General Public License.
*/

#ifndef CSTACKTRACE_H
#define CSTACKTRACE_H

#include <stdio.h>
#include <stdlib.h>
#include <execinfo.h>
#include <cxxabi.h>
#include "casyncconn.h"
#include "cserverdc.h"

/*
	this functions collects stack backtrace of the caller functions and demangles it
	then it tries to send backtrace to crash server via http
	and writes backtrace to standard error output
*/

static inline void perform_stacktrace()
{
	void* addrlist[64];
	int addrlen = backtrace(addrlist, sizeof(addrlist) / sizeof(void*));

	if (!addrlen) {
		cerr << "Stack backtrace is empty, possibly corrupt" << endl;
		return;
	}

	char** symbollist = backtrace_symbols(addrlist, addrlen);
	size_t funcnamesize = 256;
	char* funcname = (char*)malloc(funcnamesize);
	ostringstream bt;

	for (int i = 1; i < addrlen; i++) {
		char *begin_name = 0, *begin_offset = 0, *end_offset = 0;

		for (char *p = symbollist[i]; *p; ++p) {
			if (*p == '(')
				begin_name = p;
			else if (*p == '+')
				begin_offset = p;
			else if ((*p == ')') && begin_offset) {
				end_offset = p;
				break;
			}
		}

		if (begin_name && begin_offset && end_offset && (begin_name < begin_offset)) {
			*begin_name++ = '\0';
			*begin_offset++ = '\0';
			*end_offset = '\0';
			int status;
			char* ret = abi::__cxa_demangle(begin_name, funcname, &funcnamesize, &status);

			if (!status) {
				funcname = ret;
				bt << symbollist[i] << ": " << funcname << " +" << begin_offset << endl;
			} else
				bt << symbollist[i] << ": " << begin_name << "() +" << begin_offset << endl;
		} else
			bt << symbollist[i] << endl;
	}

	free(funcname);
	free(symbollist);
	cerr << "Stack backtrace:" << endl << endl << bt.str() << endl;
	cAsyncConn *http = new cAsyncConn("www.te-home.net", 80); // try to send via http

	if (!http->ok) {
		cerr << "Failed connecting to crash server, please send above stack backtrace to developers" << endl;
		http->Close();
		delete http;
		http = NULL;
		return;
	}

	ostringstream hh;
	hh << "POST /vhcs.php HTTP/1.1\n";
	hh << "Host: www.te-home.net\n";
	hh << "User-Agent: " << HUB_VERSION_NAME << '/' << VERSION << '/' << HUB_VERSION_CLASS << "\n";
	hh << "Content-Type: text/plain\n";
	hh << "Content-Length: " << bt.str().size() << "\n\n";
	hh << bt.str();
	http->Write(hh.str(), true);

	if (http->ok)
		cerr << "Successfully sent stack backtrace to crash server" << endl;
	else
		cerr << "Failed sending to crash server, please send above stack backtrace to developers" << endl;

	http->Close();
	delete http;
	http = NULL;
}

#endif // CSTACKTRACE_H
